/*
 * Copyright (c) 2008 - 2012, Andy Bierman, All Rights Reserved.
 * Copyright (c) 2012 - 2020, YumaWorks, Inc., All Rights Reserved.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *

*** Generated by yangdump-sdk

    Combined SIL module
    module example
    revision 2020-01-01
    namespace http://netconfcentral.org/ns/example
    Created: 2020-04-16T17:31:12Z
    CLI parameters:
        format c
        indent 4
        module example
        output example.c
        sil-edit2
        sil-get2
        unified true

 */

#include <xmlstring.h>

#include "procdefs.h"
#include "agt.h"
#include "agt_action.h"
#include "agt_cb.h"
#include "agt_cfg.h"
#include "agt_rpc.h"
#include "agt_sil_lib.h"
#include "agt_timer.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cfg.h"
#include "dlq.h"
#include "getcb.h"
#include "ncx.h"
#include "ncx_feature.h"
#include "ncxmod.h"
#include "ncxtypes.h"
#include "obj.h"
#include "rpc.h"
#include "status.h"
#include "val.h"
#include "val_child.h"
#include "val_util.h"
#include "xml_val.h"
#include "example.h"

/* module static variables */
static ncx_module_t *example_mod;

/* put your static variables here */

/********************************************************************
* FUNCTION y_example_init_static_vars
*
* initialize module static variables
*
********************************************************************/
static void y_example_init_static_vars (void)
{
    example_mod = NULL;

    /* init your static variables here */

} /* y_example_init_static_vars */


/********************************************************************
* FUNCTION print_callback_info
*
* Dump current Callback info
*
********************************************************************/
static void
    print_callback_info (val_value_t *useval,
                         agt_cbtyp_t cbtyp,
                         op_editop_t editop,
                         const xmlChar *callback_name)
{
    if (LOGDEBUG) {
        log_debug("\n (%s) Enter %s callback"
            "\n ------PHASE:%s;EDITOP:%s\n",
            useval ? VAL_NAME(useval) : NCX_NT_NONE,
            callback_name,
            agt_cbtype_name(cbtyp),
            op_editop_name(editop));
    }

} /* print_callback_info */


/********************************************************************
* FUNCTION navigate_in_tree
*
* This function perform the following tasks:
*
*   - Navigate up to a specific parent node
*   - Find a specific child of just found parent
*   - Run simple validation on the found node
*   - Dump Xpath representation of the val value node
*   - Log an error if special validation criteria are not met
*   - Return an error to terminate the operation
*
********************************************************************/
static status_t
    navigate_in_tree (val_value_t *useval)
{

    status_t res = NO_ERR;
    val_value_t *parent = useval->parent;

    /* The following loop will go up until the top Root is reached */
    while (parent) {

        if (obj_is_root(VAL_OBJ(parent))) {
            /* reached root */
            parent = NULL;
            break;
        } else {
            parent = parent->parent;
        }

        /* find the right parent */
        if (!xml_strcmp(VAL_NAME(parent), (const xmlChar *)"list1")) {

            /* now find desired child of this node */
            val_value_t *chval = val_get_first_child(parent);
            for ( ; chval; chval = val_get_next_child(chval)) {
                /* Refer to val.h for more API to get the first
                 * child node. For example:
                 *
                 *  - val_get_first_terminal_child()
                 *  - val_find_child()
                 *  - val_find_next_child()
                 * etc.
                 */

                if (!xml_strcmp(VAL_NAME(chval), (const xmlChar *)"A")) {
                    /* check if this child value is OK to use
                     * and generate an error if it is not acceptable
                     */

                    xmlChar *use_str = val_make_sprintf_string(chval);
                    if (!use_str) {
                        return ERR_INTERNAL_MEM;
                    }

                    /* generate an error if the value of a child os not valid */
                    if (!xml_strcmp(use_str, (const xmlChar *)"not-valid")) {

                        xmlChar *buff = NULL;
                        res =
                            val_gen_instance_id(NULL,
                                                chval,
                                                NCX_IFMT_XPATH1,
                                                &buff);
                        if (res == NO_ERR && LOGDEBUG2) {
                            log_debug2("\n    Not supported! Value of:"
                                       "\n    %sis '%s'",
                                       buff,
                                       use_str);
                        }

                        m__free(buff);
                        m__free(use_str);
                        return ERR_NCX_OPERATION_NOT_SUPPORTED;
                    }

                    m__free(use_str);
                }
            }
        }
    }

    return res;

} /* navigate_in_tree */


/********************************************************************
* FUNCTION handle_merge
*
* Dump children values during MERGE operation
* Print XPATH representation
* Navigate up and down in the val value tree and run simple validation
*
* RETURNS:
*   status, NO_ERR if successful
*
********************************************************************/
static status_t
    handle_merge (rpc_msg_t *msg,
                  val_value_t *newval,
                  val_value_t *curval)
{
    if (LOGDEBUG2) {
        log_debug("\n  Merge parameters:");
    }

    status_t res = NO_ERR;

    /* the edit is not really on this node; need to get
     * each child_undo record to get the real edited nodes
     */
    agt_cfg_transaction_t *txcb = RPC_MSG_TXCB(msg);
    agt_cfg_undo_rec_t *child_edit =
        agt_cfg_first_child_edit(txcb, newval, curval);

    while (child_edit && res == NO_ERR) {

        op_editop_t child_editop = OP_EDITOP_NONE;
        val_value_t *child_newval = NULL;
        val_value_t *child_curval = NULL;
        xmlChar *newval_str = NULL;
        xmlChar *curval_str = NULL;

        agt_cfg_child_edit_fields(child_edit,
                                  &child_editop,
                                  &child_newval,
                                  &child_curval);

        if (child_newval) {
            newval_str = val_make_sprintf_string(child_newval);
            if (!newval_str) {
                return ERR_INTERNAL_MEM;
            }
        }

        if (child_curval) {
            curval_str = val_make_sprintf_string(child_curval);
            if (!curval_str) {
                if (newval_str) {
                    m__free(newval_str);
                }
                return ERR_INTERNAL_MEM;
            }
        }

        if (LOGDEBUG2) {
            log_debug2("\n    %s: editop=%s newval=%s curval=%s",
                       child_newval ? VAL_NAME(child_newval) : NCX_EL_NULL,
                       child_editop ? op_editop_name(child_editop) : NCX_EL_NONE,
                       child_newval ? newval_str : NCX_EL_NULL,
                       child_curval ? curval_str : NCX_EL_NULL);
        }

        xmlChar *buff = NULL;
        val_value_t *logval = child_newval ? child_newval : child_curval;
        if (logval) {

            /* Generate XPATH full path for this child */
            res =
                val_gen_instance_id(NULL,
                                    logval,
                                    NCX_IFMT_XPATH1,
                                    &buff);
            if (res == NO_ERR && LOGDEBUG2) {
                log_debug2("\n    Child '%s' XPATH: %s",
                           VAL_NAME(logval),
                           buff);
            }
        }

        if (child_newval && val_set_by_default(child_newval)) {
            if (LOGDEBUG2) {
                log_debug2("\n        NEWVAL is default");
            }
        }

        if (child_curval && val_set_by_default(child_curval)) {
            if (LOGDEBUG2) {
                log_debug2("\n        CURVAL is default");
            }
        }

        res = navigate_in_tree(logval);
        if (res != NO_ERR) {
            /* report an error */
        }

        m__free(buff);
        m__free(newval_str);
        m__free(curval_str);

        child_edit = agt_cfg_next_child_edit(child_edit);
    }

    return res;

} /* handle_merge */


/********************************************************************
* FUNCTION example_edit2_cb
*
* Edit database object callback
*
* Path: list /top-cont
* Path: list /top-cont/list1
* Path: list /top-cont/list1/inner
*
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    example_edit2_cb (ses_cb_t *scb,
                      rpc_msg_t *msg,
                      agt_cbtyp_t cbtyp,
                      op_editop_t editop,
                      val_value_t *newval,
                      val_value_t *curval)
{
    status_t res = NO_ERR;
    val_value_t *errorval = (curval) ? curval : newval;
    uint32 cur_errs = agt_get_error_count(msg);

    print_callback_info(errorval, cbtyp, editop, (const xmlChar *)"EDIT-2");

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_MERGE:
            res = handle_merge(msg, newval, curval);
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }
        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }

    if ((res != NO_ERR) && (cur_errs == agt_get_error_count(msg))) {
        agt_record_error(scb,
                         &msg->mhdr,
                         NCX_LAYER_CONTENT,
                         res,
                         NULL,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval);
    }
    return res;

} /* example_edit2_cb */


/********************************************************************
* FUNCTION y_example_init
*
* initialize the example server instrumentation library
*
* INPUTS:
*    modname == requested module name
*    revision == requested version (NULL for any)
*
* RETURNS:
*     error status
********************************************************************/
status_t y_example_init (const xmlChar *modname,
                         const xmlChar *revision)
{
    status_t res = NO_ERR;

    y_example_init_static_vars();

    /* change if custom handling done */
    if (xml_strcmp(modname, y_example_M_example)) {
        return ERR_NCX_UNKNOWN_MODULE;
    }

    if (revision && xml_strcmp(revision, y_example_R_example)) {
        return ERR_NCX_WRONG_VERSION;
    }

    res =
        ncxmod_load_module(y_example_M_example,
                           y_example_R_example,
                           agt_get_savedevQ(),
                           &example_mod);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_edit2_callback(y_example_M_example,
                                       (const xmlChar *)"/ex:top-cont",
                                       y_example_R_example,
                                       example_edit2_cb);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_edit2_callback(y_example_M_example,
                                       (const xmlChar *)"/ex:top-cont/ex:list1",
                                       y_example_R_example,
                                       example_edit2_cb);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_edit2_callback(y_example_M_example,
                                       (const xmlChar *)"/ex:top-cont/ex:list1/ex:inner",
                                       y_example_R_example,
                                       example_edit2_cb);
    if (res != NO_ERR) {
        return res;
    }

    /* put your module initialization code here */

    return res;

} /* y_example_init */


/********************************************************************
* FUNCTION y_example_init2
*
* SIL init phase 2: non-config data structures
* Called after running config is loaded
*
* RETURNS:
*     error status
********************************************************************/
status_t y_example_init2 (void)
{
    status_t res = NO_ERR;

    /* put your init2 code here */

    return res;

} /* y_example_init2 */

/********************************************************************
* FUNCTION y_example_cleanup
*    cleanup the server instrumentation library
*
********************************************************************/
void y_example_cleanup (void)
{

    agt_cb_unregister_callbacks(y_example_M_example,
                                (const xmlChar *)"/ex:top-cont");

    agt_cb_unregister_callbacks(y_example_M_example,
                                (const xmlChar *)"/ex:top-cont/ex:list1");

    agt_cb_unregister_callbacks(y_example_M_example,
                                (const xmlChar *)"/ex:top-cont/ex:list1/ex:inner");

    /* put your cleanup code here */

} /* y_example_cleanup */


/* END SIL example.c */
